import { createPlugin, createToken } from 'fusion-core';
import { FetchToken } from 'fusion-tokens';
import { GraphQLSchemaToken, ApolloContextToken } from 'fusion-apollo';
import { ApolloClient } from 'apollo-client';
import { HttpLink } from 'apollo-link-http';
import { ApolloLink, from } from 'apollo-link';
import { SchemaLink } from 'apollo-link-schema';
import { InMemoryCache } from 'apollo-cache-inmemory';
import { get } from 'js-cookie';

/** Copyright (c) 2018 Uber Technologies, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
// Fixed By: https://github.com/benmosher/eslint-plugin-import/issues/975#issuecomment-348807796
// eslint-disable-next-line
const GetApolloClientCacheToken = createToken('GetApolloClientCacheToken');
const ApolloClientCredentialsToken = createToken('ApolloClientCredentialsToken');
const ApolloClientEndpointToken = createToken('ApolloClientEndpointToken');
const ApolloClientDefaultOptionsToken = createToken('ApolloClientDefaultOptionsToken');
const GetApolloClientLinksToken = createToken('GetApolloClientLinksToken');
const ApolloClientAuthKeyToken = createToken('ApolloClientAuthKeyToken');

function Container() {}

const ApolloClientPlugin = createPlugin({
  deps: {
    getCache: GetApolloClientCacheToken.optional,
    endpoint: ApolloClientEndpointToken.optional,
    fetch: FetchToken,
    includeCredentials: ApolloClientCredentialsToken.optional,
    authKey: ApolloClientAuthKeyToken.optional,
    apolloContext: ApolloContextToken.optional,
    getApolloLinks: GetApolloClientLinksToken.optional,
    schema: GraphQLSchemaToken.optional,
    defaultOptions: ApolloClientDefaultOptionsToken.optional
  },

  provides({
    getCache = ctx => new InMemoryCache(),
    endpoint = '/graphql',
    fetch,
    authKey = 'token',
    includeCredentials = 'same-origin',
    apolloContext = ctx => ctx,
    getApolloLinks,
    schema,
    defaultOptions
  }) {
    function getClient(ctx, initialState) {
      const cache = getCache(ctx);

      const getBrowserProps = () => {
        return get(authKey);
      };

      const connectionLink = schema && false ? new SchemaLink({
        schema,
        context: typeof apolloContext === 'function' ? // $FlowFixMe
        apolloContext(ctx) : apolloContext
      }) : new HttpLink({
        uri: endpoint,
        credentials: includeCredentials,
        fetch
      });
      const token = getBrowserProps();
      const authMiddleware = new ApolloLink((operation, forward) => {
        if (token) {
          operation.setContext({
            headers: {
              authorization: `Bearer ${token}`
            }
          });
        }

        return forward(operation);
      });
      const links = getApolloLinks ? getApolloLinks([authMiddleware, connectionLink], ctx) : [authMiddleware, connectionLink];
      const client = new ApolloClient({
        ssrMode: false,
        connectToDevTools: true && process.env.NODE_ENV !== "production",
        link: from(links),
        cache: cache.restore(initialState),
        defaultOptions
      });
      return client;
    }

    return (ctx, initialState) => {
      if (ctx.memoized.has(Container)) {
        return ctx.memoized.get(Container);
      }

      const client = getClient(ctx, initialState);
      ctx.memoized.set(Container, client);
      return client;
    };
  }

});

export default ApolloClientPlugin;
export { GetApolloClientCacheToken, ApolloClientCredentialsToken, ApolloClientEndpointToken, ApolloClientDefaultOptionsToken, GetApolloClientLinksToken, ApolloClientAuthKeyToken };
//# sourceMappingURL=browser.es2017.es.js.map
